#include "MarcoPolo.h"

// NON const static var declarations
MarcoPolo::netBufStat MarcoPolo::bufStat;
char MarcoPolo::netBuf[2];

#ifdef TGT_ERS7
#include "Shared/ERS7Info.h"
#define IRSENSOROFFSET FarIRDistOffset
#else
#include "Shared/ERS220Info.h"
#define IRSENSOROFFSET IRDistOffset
#endif

#include "Shared/Config.h"

#include "Events/TextMsgEvent.h"
#include "Shared/WorldState.h"

#include "Motion/MMAccessor.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WaypointWalkMC.h"

#include <math.h>

#define NEAR_THRESHHOLD 700

//better to put this here instead of the header
using namespace std; 

MarcoPolo::MarcoPolo() : BehaviorBase("MarcoPolo"),
	LMDetector(new LandmarkDetectionGenerator()),
	dest(0),
	myLocation(0),
	otherLocation(0),
	facingCW(false),
	traveling(false),
	headpointer_id(MotionManager::invalid_MC_ID),
	walker_id(MotionManager::invalid_MC_ID),
	waypt_walker_id(MotionManager::invalid_MC_ID),
	lowpos(RobotInfo::outputRanges[HeadOffset+PanOffset][MinRange]),
	highpos(RobotInfo::outputRanges[HeadOffset+PanOffset][MaxRange]),
	midpos((lowpos+highpos)/2),
	lastLeft(0),
	lastRight(0),
	lastMid(0),
	motNumber(0),
	moveDir(0),
	walkBusy(false),
	shouldTurn180(false),
	connected(false),
	newConnection(true)
	 {

	// Setup the map
	map[0]  = new MapNode(PINK, GREEN, YELLOW, BLUE, 0, 400);
	map[1]  = new MapNode(BLUE, YELLOW, GREEN, PINK, 400, 400, map[0]);
	map[2]  = new MapNode(BLUE, YELLOW, PINK, GREEN, 400, 400, map[1]);
	map[3]  = new MapNode(BLUE, GREEN, YELLOW, PINK, 400, 400, map[2]);
	map[4]  = new MapNode(BLUE, GREEN, PINK, YELLOW, 400, 400, map[3]);
	map[5]  = new MapNode(BLUE, PINK, YELLOW, GREEN, 400, 400, map[4]);
	map[6]  = new MapNode(BLUE, PINK, GREEN, YELLOW, 400, 400, map[5]);
	map[7]  = new MapNode(YELLOW, BLUE, GREEN, PINK, 400, 400, map[6]);
	map[8]  = new MapNode(YELLOW, BLUE, PINK, GREEN, 400, 400, map[7]);
	map[9]  = new MapNode(YELLOW, GREEN, BLUE, PINK, 400, 400, map[8]);
	map[10] = new MapNode(YELLOW, GREEN, PINK, BLUE, 400, 400, map[9]);
	map[11] = new MapNode(YELLOW, PINK, BLUE, GREEN, 400, 400, map[10]);
	map[12] = new MapNode(YELLOW, PINK, GREEN, BLUE, 400, 400, map[11]);
	map[13] = new MapNode(GREEN, BLUE, YELLOW, PINK, 400, 400, map[12]);
	map[14] = new MapNode(GREEN, BLUE, PINK, YELLOW, 400, 400, map[13]);
	map[15] = new MapNode(GREEN, YELLOW, BLUE, PINK, 400, 400, map[14]);
	map[16] = new MapNode(GREEN, YELLOW, PINK, BLUE, 400, 400, map[15]);
	map[17] = new MapNode(GREEN, PINK, BLUE, YELLOW, 400, 400, map[16]);
	map[18] = new MapNode(GREEN, PINK, YELLOW, BLUE, 400, 400, map[17]);
	map[19] = new MapNode(PINK, BLUE, YELLOW, GREEN, 400, 400, map[18]);
	map[20] = new MapNode(PINK, BLUE, GREEN, YELLOW, 400, 400, map[19]);
	map[21] = new MapNode(PINK, YELLOW, BLUE, GREEN, 400, 400, map[20]);
	map[22] = new MapNode(PINK, YELLOW, GREEN, BLUE, 400, 400, map[21]);
	map[23] = new MapNode(PINK, GREEN, BLUE, YELLOW, 400, 400, map[21]);
	for(int i=0; i<23; i++) map[i]->setNext(map[i+1]);
	
	// set up the client / server info
	#ifdef TGT_ERS7
	MarcoPolo::myTask = server;
	#else
	MarcoPolo::myTask = slave;
	#endif
	
	bufStat = OLD;
	
}

MarcoPolo::~MarcoPolo() {
	for(int i=0; i<24; i++) delete map[i];
}


void MarcoPolo::DoStart() {
	BehaviorBase::DoStart(); // do this first (required)

	// Socket communication DoStart
	if (myTask == server)
		SetupServer();
	else
		SetupSlave();
		
	bufStat = OLD;
		
	// Landmark Detection DoStart
	LMDetector->DoStart();
	erouter->addListener(this,EventBase::visObjEGID);
	erouter->addListener(this, EventBase::textmsgEGID);
	
	// Hallway Navigation DoStart
	
	headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
	walker_id = motman->addPersistentMotion(SharedObject<WalkMC>());
	
	walkBusy = false;
	lastLeft = -1U;
	lastRight = -1U;
	lastMid = -1U;
		
	// add timer to handle walking
	//tick every second
	erouter->addTimer(this,TIMER_WALKTICK,1000,true);
		
	motNumber = 0;
	// hopefully this is not too fast.
	// start the head panning
	MMAccessor<HeadPointerMC>(headpointer_id)->setMaxSpeed(PanOffset,2);
	MMAccessor<HeadPointerMC>(headpointer_id)->setJoints(tiltTop,midpos,headNod);
	erouter->addListener(this,EventBase::motmanEGID,headpointer_id,EventBase::statusETID); // Listen for head motion completion

}

void MarcoPolo::DoStop() {	
	erouter->removeListener(this); //generally a good idea, unsubscribe all
	// Socket communication DoStop
	if (myTask == server)
		ShutdownServer();
	else
		ShutdownSlave();
		
	LMDetector->DoStop();
	
	motman->removeMotion(walker_id);
	motman->removeMotion(headpointer_id);
	
	BehaviorBase::DoStop(); // do this last (required)
}

void MarcoPolo::processEvent(const EventBase& e) {
	// Landmark Detection
	if(e.getSourceID() == LawrenceInterface::visLMSID) {
		facingCW = (state->outputs[HeadOffset+PanOffset] < 0);
		short i;
		// Identify the landmark
		for(i=0; i<24; i++) if(*(map[i]) == *(LMDetector->getNode())) break;
		printf("Saw landmark %X\n", i);
		myLocation = i;
		if(myLocation == dest) traveling = false; // Are we there yet?
		else if(shouldWalkCW() != facingCW) turn180(); // Turn arround if need be
	}
	
	// Hall navigation related events
	else if(e.getGeneratorID() == EventBase::motmanEGID) {
		if ((e.getSourceID() == waypt_walker_id)) {
			// for now, this should never happen.. nothing is subscribing to these events
		} else {
			float IRDist = 0.0;
			bool isRight = false;
			switch(motNumber) {
			case 0:
				IRDist = state->sensors[IRSENSOROFFSET];
				std::cout << "LocateDestination: mot" << motNumber << "(0) MIDPOS" << " dist: " << IRDist << endl;
				MMAccessor<HeadPointerMC>(headpointer_id)->setJoints(tiltTop,lowpos,headNod);
				moveDir = 0;
				motNumber++;
				lastMid = IRDist;
				break;
			case 1:
				if (moveDir == 0) isRight = true;
				else isRight = false;
				IRDist = state->sensors[IRSENSOROFFSET];
				std::cout << "LocateDestination: mot" << motNumber << (isRight ? "(1) RIGHTPOS" : "(1) LEFTPOS") << " dist: " << IRDist << endl;
				MMAccessor<HeadPointerMC>(headpointer_id)->setJoints(tiltTop,midpos,headNod);
				isRight ? motNumber++ : motNumber--;	
				isRight ? lastRight = IRDist : lastLeft = IRDist;
				break;
			case 2:
				IRDist = state->sensors[IRSENSOROFFSET];
				std::cout << "LocateDestination: mot" << motNumber << " (2) MIDPOS" << " dist: " << IRDist << endl;
				MMAccessor<HeadPointerMC>(headpointer_id)->setJoints(tiltTop,highpos,headNod);
				moveDir = 1;
				motNumber--;
				lastMid = IRDist;
				break;
			}
		}
	}
	else if (e.getGeneratorID() == EventBase::timerEGID) {
		if (e.getSourceID() == TIMER_WALKTICK) walk_ticker();
		else if (e.getSourceID() == TIMER_ROTATEWAIT) {
			//clean up -- this is the timer that fires after the turn has executed
			cout << "Cleaning up from rotation" << endl;
			motman->removeMotion(waypt_walker_id);
			walkBusy = false;
		} else if (e.getSourceID() == TIMER_180WAIT) {
			//clean up -- this is the timer that fires after the 180 turn has executed
			cout << "Cleaning up from 180 rotation" << endl;
			motman->removeMotion(waypt_walker_id);
			shouldTurn180 = false;
			walkBusy = false;
		} else if (e.getSourceID() == TIMER_NETTICK) {
			(myTask == server) ? TickServer() : TickClient();
		}
	}
	
	// User command
	else if(e.getGeneratorID() == EventBase::textmsgEGID) {
		const char* msgTxt = (dynamic_cast<const TextMsgEvent&>(e)).getText().c_str();
		//const TextMsgEvent &txtE = *reinterpret_cast<const TextMsgEvent*>(&e);
		// Parse text message event
		if(strncmp(msgTxt, "goto ", 5) == 0) {
			const unsigned char goal = atoi(msgTxt + 5);
			printf("New destination is: %X\n", goal);
			dest = goal;
			traveling = true; // Start walking
		} else if (strncmp(msgTxt, "send ", 5) == 0) {
			theSock->printf(msgTxt+5);
		}
		delete msgTxt;
	}
}


void MarcoPolo::walk_ticker() {
	if (walkBusy)
	{
		// do nothing! hooray!
		cout << "walk is busy" << endl;
	} else if(traveling) {
		//check the front sensor
			
		
		if (shouldTurn180) {
				cout << "turning 180...\n";
				
				//set walkbusy
				walkBusy = true;
				
				//excecute turn
				waypt_walker_id = motman->addPersistentMotion(SharedObject<WaypointWalkMC>());
				MMAccessor<WaypointWalkMC>(waypt_walker_id)->setIsLooping(false);
				MMAccessor<WaypointWalkMC>(waypt_walker_id)->addEgocentricWaypoint(0,0,M_PI,false,.075);
				
				MMAccessor<WaypointWalkMC>(waypt_walker_id)->go();
					
				//ugly hack!
				erouter->addTimer(this,TIMER_180WAIT,5000,false);
		// make sure that all sensors at least have SOME data.. 
		} else if ( !( (lastMid < 0) || (lastRight < 0) || (lastLeft < 0) ) )
		{
			if (lastMid <= NEAR_THRESHHOLD)
			{
				cout << "alignment case" << endl;
				//execute turning sequence
				
				// THIS IS NECESSARY
				MMAccessor<WalkMC>(walker_id)->setTargetVelocity(0,0,0);
				walkBusy = true;
					
				// compute stuff -- comments later LOL INTERNET!!!
				float hyp = 0.0;
				float rotate_angle = 0.0;
					
				//figure out the angle to rotate, and then DOOO IT!
					
				if (lastRight < lastLeft)
				{
					hyp = sqrt( pow(lastMid,2) + pow(lastRight,2) );
					rotate_angle = (M_PI) - (M_PI/2) - acos( lastRight / hyp );
				} else if (lastRight > lastLeft) {
					hyp = sqrt( pow(lastMid,2) + pow(lastLeft,2) );
					rotate_angle = -1 * ( (M_PI) - (M_PI/2) - acos( lastLeft / hyp ));
				} else {
					rotate_angle = shouldWalkCW() ? (M_PI/2) : -1 * (M_PI/2);
				}
					
				cout << "\n\n ROTATION EVENT \n\n";
				cout << "lastleft: " << lastLeft << endl << "lastright: " << lastRight << endl << "hyp: " << hyp << " angle: " << rotate_angle << endl;
				//excecute turn
				waypt_walker_id = motman->addPersistentMotion(SharedObject<WaypointWalkMC>());
				MMAccessor<WaypointWalkMC>(waypt_walker_id)->setIsLooping(false);
				MMAccessor<WaypointWalkMC>(waypt_walker_id)->addEgocentricWaypoint(0,0,rotate_angle,false,.075);
				
				MMAccessor<WaypointWalkMC>(waypt_walker_id)->go();
					
				//ugly hack!
				//erouter->addListener(this,EventBase::motmanEGID,waypt_walker_id); // Listen for rotation complete
				erouter->addTimer(this,TIMER_ROTATEWAIT,5000,false);
					
			} else {
				// walk straight forward!
				cout << "walk forward case" << endl;
				MMAccessor<WalkMC>(walker_id)->setTargetVelocity(100,0,0,-1);
			}
		}
	}
}

bool MarcoPolo::shouldWalkCW() {
	return dest > myLocation;
}

void MarcoPolo::turn180() {
	shouldTurn180 = true;
}

void MarcoPolo::SetupServer()
{
	cout << "I am the server" << endl;
	// Setup wireless communication
	theSock = wireless->socket(SocketNS::SOCK_STREAM);
	wireless->setReceiver(theSock->sock, processWireless);
	wireless->setDaemon(theSock,true);
	wireless->listen(theSock->sock, 10101);
	
	// 5 second repeat for checking communication
	erouter->addTimer(this,TIMER_NETTICK,5000,true);
	erouter->addListener(this,EventBase::buttonEGID);
}

void MarcoPolo::SetupSlave()
{
	cout << "I am the slave" << endl;
	// try to connect and print "something"
	theSock=wireless->socket(SocketNS::SOCK_STREAM);

	wireless->setReceiver(theSock->sock, processWireless);
	//connect
	wireless->connect(theSock,"10.0.137.7",10101);
	// 5 second repeat for checking communication
	erouter->addTimer(this,TIMER_NETTICK,5000,true);
}

void MarcoPolo::ShutdownServer()
{
	wireless->setDaemon(theSock,false);
	wireless->close(theSock);
}

void MarcoPolo::ShutdownSlave()
{
	wireless->close(theSock);
}

void MarcoPolo::TickServer()
{
	connected = wireless->isConnected(theSock->sock);
	
	if (connected && newConnection)
	{
		cout << "New connection established. " << endl;
		newConnection = false;
	}
	
	if (bufStat == NEW)
	{
		cout << "TICKSERVER: NEW DATA: " << *netBuf << endl;
		bufStat = OLD;
	}
}

void MarcoPolo::TickClient()
{
	connected = wireless->isConnected(theSock->sock);
	
	if (connected && newConnection)
	{
		cout << "New connection established. " << endl;
		newConnection = false;
	}
	
	//if at first you don't connect; connect, connect again!
	//will this cause problems? who can say?
	if (connected == false)
	{
		cout << "Trying connection to server... " << endl;
		wireless->connect(theSock,"10.0.137.7",10101);
	}
	if (bufStat == NEW)
	{
		cout << "TICKCLIENT: NEW DATA: " << *netBuf << endl;
		bufStat = OLD;
	}
}

int MarcoPolo::processWireless(char *buf, int bytes) {
	cout << "processWireless Received\t";
	
	for(int i=0; i<bytes; i++) 
	{
		cout << buf[i];
		netBuf[i] = buf[i];
	}
	cout << endl;
	
	// set bufStat flag so that it knows new info has been received.
	bufStat = NEW;
	return 0;
}

/*! @file
 * $Author BHW & DTC $
 * $Name MarcoPolo main behavior $
 * $Revision 0.3 $
 * $State Development $
 * $Date 2006-05-13 $
 */


