//-*-c++-*-
#ifndef INCLUDED_MARCO_POLO_h_
#define INCLUDED_MARCO_POLO_h_

#include "Behaviors/BehaviorBase.h"
#include "Events/EventRouter.h"

#include "Motion/MotionManager.h"

#include "Wireless/Wireless.h"

#include "detectors/LandmarkDetectionGenerator.h"
#include "mapNode.h"

//! DESCRIPTION
class MarcoPolo : public BehaviorBase {
public:
	MarcoPolo();
	~MarcoPolo();

	virtual void DoStart();
	virtual void DoStop();
	// called by erouter when an event we're subscribed to occures
	virtual void processEvent(const EventBase& e);
	static int processWireless(char *buf, int bytes);	
	static std::string getClassDescription() { return "Lawrence University Marco Polo Behavior"; }
	virtual std::string getDescription() const { return getClassDescription(); }
	
	enum commTask { server,slave };
	enum netBufStat { OLD,NEW };
	
protected:
	LandmarkDetectionGenerator* LMDetector;
	MapNode* map[24];
	unsigned char dest, myLocation, otherLocation;
	bool facingCW;
	bool traveling;
	
	MotionManager::MC_ID headpointer_id; // a HeadPointerMC object
	MotionManager::MC_ID walker_id; // walker, yall
	MotionManager::MC_ID waypt_walker_id;
	
	Socket *theSock;
	
	commTask myTask;
	static netBufStat bufStat;
	static char netBuf[2];

	static const float tiltTop = 0.0;
	static const float headNod = 0.0;
	
	//timer IDS
	static const int TIMER_WALKTICK = 0;
	static const int TIMER_ROTATEWAIT = 1;
	static const int TIMER_180WAIT = 2;
	static const int TIMER_NETTICK = 3;
	
	const float lowpos;
	const float highpos;
	const float midpos;
	
	float lastLeft;
	float lastRight;
	float lastMid;
	
	short motNumber;
	short moveDir;
	
	bool walkBusy;
	bool shouldTurn180;
	bool connected;
	bool newConnection;
		

	//startup code for larry
	virtual void SetupServer();
	//startup code for cody
	virtual void SetupSlave();
	//shutdown code for larry
	virtual void ShutdownServer();
	//shutdown code for cody
	virtual void ShutdownSlave();
	//timer code for larry
	virtual void TickServer();
	//timer code for cody
	virtual void TickClient();
	
	/*! executes whenever the walkticker timer ticks
	 * going to use landmarks, somehow
	 * walk forward unless an object is detected in front of the robot
	 * in that case, turn to align straight
	 * right now, ignore the fact that sensor data isn't necessarily current
	 */
	void walk_ticker();
	
	/*! Returns whether we should walk clockwise arround the hall
	 * based on the current location and the destination. False indicates
	 * that we should go counter-clockwise.
	 */
	bool shouldWalkCW();
	void turn180();
	
	
	
private:
	// Providing declarations for these functions will avoid a compiler warning if
	// you have any class members which are pointers.  However, as it is, an error
	// will result if you inadvertantly cause a call to either (which is probably
	// a good thing, unless you really intended to copy/assign a behavior, in
	// which case simply provide implementations for the functions)
	MarcoPolo(const MarcoPolo&); //!< don't call (copy constructor)
	MarcoPolo& operator=(const MarcoPolo&); //!< don't call (assignment operator)
};

/*! @file
 * @brief Defines MarcoPolo, which is a behavior to be run simulteniously on two
 * or more AIBOs. They attempt to meet up with each other through localizing
 * themselves on a stored map and communicating the location they believe them-
 * selves to be in and agreeing on a rendevous location.
 * @author Casner, D. and Willard, B. (Creator)
 *
 * $Author BHW & DTC $
 * $Name MarcoPolo main behavior $
 * $Revision 0.3 $
 * $State Development $
 * $Date 2006-05-13 $
 */

#endif
