#include "comm_demo.h"
#include "Shared/ERS7Info.h";

//better to put this here instead of the header
using namespace std; 


static comm_demo* theOne = NULL;


comm_demo::comm_demo() : BehaviorBase("comm_demo") {
	
#ifdef TGT_ERS220
comm_demo::myTask = slave;
#endif

#ifdef TGT_ERS7
comm_demo::myTask = server;
#endif
	
	if(theOne != NULL) theOne = this;
	serr->printf("Already a comm_demo behavior running!");
}

comm_demo::~comm_demo() {
	if(theOne == this) theOne = NULL;	
}


void comm_demo::DoStart() {
	BehaviorBase::DoStart(); // do this first (required)
	if (myTask == server)
		SetupServer();
	else
		SetupSlave();
}

void comm_demo::DoStop() {
	// <your shutdown code here>
	if (myTask == server)
		ShutdownServer();
	else
		ShutdownSlave();
	BehaviorBase::DoStop(); // do this last (required)
}

void comm_demo::processEvent(const EventBase& e) {
	switch (e.getGeneratorID())
	{
		case EventBase::timerEGID:
			(myTask == server) ? TickServer() : TickClient();
			break;
		case EventBase::buttonEGID:
			cout << "Button Pressed: " << e.getSourceID() << endl;
			if (e.getSourceID() == ERS7Info::HeadButOffset)
			{
				theSock->printf("1");
			}
			else
			{
				// for debug
				cout << "Socket Connected: " << wireless->isConnected(theSock->sock) << endl;
				cout << "Socket Ready: " << wireless->isReady(theSock->sock) << endl;
				//
				theSock->printf("0");
			}
			break;
		default:
			cout << "Unknown Event: " << e.getName() << endl;
			break;
	}
}

void comm_demo::SetupServer()
{
	cout << "I am the server" << endl;
	// Setup wireless communication
	theSock = wireless->socket(SocketNS::SOCK_STREAM);
	wireless->setDaemon(theSock,true);
	wireless->listen(theSock->sock, 10101);
	
	// 5 second repeat for checking communication
	erouter->addTimer(this,0,5000,true);
	erouter->addListener(this,EventBase::buttonEGID);
}

void comm_demo::SetupSlave()
{
	cout << "I am the slave" << endl;
	// try to connect and print "something"
	theSock=wireless->socket(SocketNS::SOCK_STREAM);

	//connect
	wireless->connect(theSock,"10.0.137.7",10101);
	erouter->addTimer(this,0,32,true);
}

void comm_demo::ShutdownServer()
{
	wireless->setDaemon(theSock,false);
	wireless->close(theSock);
}

void comm_demo::ShutdownSlave()
{
	wireless->close(theSock);
}

void comm_demo::TickServer()
{
	// nothing yet
}

void comm_demo::TickClient()
{
	// nothing yet
}

/*! @file
 * $Author BHW & DTC $
 * $Name commdemo main behavior $
 * $Revision 0.01 $
 * $State Development $
 * $Date 2006-03-31 $
 */

