//-*-c++-*-
#ifndef INCLUDED_MAPNODE_h_
#define INCLUDED_MAPNODE_h_

#include "local_interface.h"

enum LMColors {
	PINK,
	BLUE,
	YELLOW,
	GREEN
};

//! Map node class for the MarcoPolo behavior
class MapNode {
protected:
	LMColors arrangement[4];
	unsigned int forward;
	unsigned int back;
	MapNode* next;
	MapNode* prev;
	
public:
	MapNode(LMColors UL, LMColors UR, LMColors LL, LMColors LR,
			unsigned int forwardDist, unsigned int backDist,
			MapNode* nextNode = NULL, MapNode* prevNode = NULL) : 
		forward(forwardDist),
		back(backDist),
		next(nextNode),
		prev(prevNode) {
		arrangement[0] = UL;
		arrangement[1] = UR;
		arrangement[2] = LL;
		arrangement[3] = LR;
		}

	MapNode(unsigned int forwardDist, unsigned int backDist,
			MapNode* nextNode = NULL, MapNode* prevNode = NULL) : 
		forward(forwardDist),
		back(backDist),
		next(nextNode),
		prev(prevNode) {}
		
	MapNode(LMColors UL, LMColors UR, LMColors LL, LMColors LR) :
		forward(0),
		back(0),
		next(NULL),
		prev(NULL) {
		arrangement[0] = UL;
		arrangement[1] = UR;
		arrangement[2] = LL;
		arrangement[3] = LR;
		}
	
	void setNext(MapNode* nextNode) {
		next = nextNode;
	}
	
	void setPrev(MapNode* prevNode) {
		prev = prevNode;	
	}
	
	void setArrangement(LMColors UL, LMColors UR, LMColors LL, LMColors LR) {
		arrangement[0] = UL;
		arrangement[1] = UR;
		arrangement[2] = LL;
		arrangement[3] = LR;
	}
	
	LMColors getArrangment(int x, int y) {
		return arrangement[x+2*y];
	}
	
	unsigned int getFor() {
		return forward;
	}
	unsigned int getBack() {
		return back;
	}
	
	MapNode& operator++() {
		return *next;
	}
	MapNode& operator--() {
		return *prev;
	}
	
	bool operator==(const MapNode& otherNode) {
		for(int i=0; i<4; i++)
			if(arrangement[i] != otherNode.arrangement[i])
				return false;
		// If we made it through the loop then they are equal
		return true;
	}
	
	char getColorLetter(int x, int y) {
		switch(getArrangment(x, y)) {
			case PINK:
				return 'P';
			case BLUE:
				return 'B';
			case YELLOW:
				return 'Y';
			case GREEN:
				return 'G';	
		}
		return -1;
	}
	
	void print() {
		printf("\t%c %c\n\t%c %c\n",
		       getColorLetter(0,0),
		       getColorLetter(1,0),
		       getColorLetter(0,1),
		       getColorLetter(1,1));
	}
	
private:
	// Do not call, provided to eliminate compiler warnings
	MapNode(const MapNode&);
	MapNode& operator=(const MapNode&);
};

/*! @file
 * @brief Map node class for the MarcoPolo behavior
 * @author DTC (Creator)
 *
 * $Author Daniel Casner$
 * $Name MapNode$
 * $Revision 0.01$
 * $State Development$
 * $Date 2006-04-22$
 */

#endif
